/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSSLASN1TIME_H_
#define FANCYSSLASN1TIME_H_

#include <fancysslbase.h>
#include <time.h>

extern "C" {
	struct asn1_string_st;
	typedef struct asn1_string_st ASN1_TIME;
}

namespace Fancy {

namespace SSL {

class FANCYSSL_API ASN1Time: public Fancy::SSL::Base {

public:
	typedef Fancy::ObjPtr<ASN1Time> ptr_t;

	static ptr_t newInstance(ASN1_TIME *asn_time);
	static ptr_t newInstance(time_t t);

	ASN1_TIME *getASN1_TIME() const;
	ASN1_TIME *dup() const;

	int compare(time_t t) const;
	int compare(const ASN1Time &t) const;

	bool lessThan(const Fancy::Object &o) const;
	bool equals(const Fancy::Object &o) const;

	time_t toTime_t(int *tz_off) const;
	std::string toString() const;

protected:
	ASN1Time(ASN1_TIME *_asn1_time);
	virtual ~ASN1Time();

private:
	ASN1_TIME *asn1_time;
};

inline bool operator< (const ASN1Time &a, const ASN1Time &b) {
	return a.compare(b) < 0;
}

inline bool operator== (const ASN1Time &a, const ASN1Time &b) {
	return a.compare(b) == 0;
}

inline bool operator!= (const ASN1Time &a, const ASN1Time &b) {
	return a.compare(b) != 0;
}

inline bool operator< (const ASN1Time &a, time_t b) {
	return a.compare(b) < 0;
}

inline bool operator== (const ASN1Time &a, time_t b) {
	return a.compare(b) == 0;
}

inline bool operator!= (const ASN1Time &a, time_t b) {
	return a.compare(b) != 0;
}
}
}

#endif /* FANCYSSLASN1TIME_H_ */
