/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** Base functionality for OpenSSL integration.
**
*****************************************************************************/

#ifndef FANCYSSL_BASE_H
#define FANCYSSL_BASE_H

#ifdef WIN32
# ifdef __GNUC__
#  ifdef BUILD_fancyssl3_DLL
#   define FANCYSSL_API __attribute__((dllexport))
#  else
#   define FANCYSSL_API __attribute__((dllimport))
#  endif
# else
#  ifdef BUILD_fancyssl3_DLL
#   define FANCYSSL_API __declspec(dllexport)
#  else
#   define FANCYSSL_API __declspec(dllimport)
#  endif
# endif
#else
# ifdef __GNUC__
#  define FANCYSSL_INTERNAL_API __attribute__((visibility("hidden")))
# endif
#endif

#ifndef FANCYSSL_INTERNAL_API
#define FANCYSSL_INTERNAL_API
#endif

#ifndef FANCYSSL_API
#define FANCYSSL_API
#endif

#include <fancyobject.h>

namespace Fancy {
  namespace SSL {

  /**
   * This is the base calss for all objects, which encapsulate an OpenSSL
   * handle. The constructor of this class ensures, that the openssl library is
   * correctly initialized.
   */
  class FANCYSSL_API Base : virtual public Fancy::Object
  {
  public:
	  typedef Fancy::ObjPtr<Base> ptr_t;

	  /**
	   * Perform one-time initializations, if not done so far.
	   * This is needed for static functionality, which is executed before
	   * actually calling the Base ctor. This method my be called as often as you
	   * want and from concurrent threads. It is guaranteed, that the initializations
	   * are only performed once.
	   */
	  static void init();

	  /**
	   * Return the last formatted openssl error message which occurred inside this thread.
	   */
	  static std::string errorMessage();

  protected:

	  /**
	   * Perform one-time initializations by calling @c Base::init()
	   */
	  Base();
	  virtual ~Base();
  };
  }
}

#endif
