/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSSLBIGNUM_H_
#define FANCYSSLBIGNUM_H_

#include <fancysslbase.h>
#include <fancyarray.h>

extern "C" {
	struct bignum_st;
	typedef struct bignum_st BIGNUM;
	struct asn1_string_st;
	typedef struct asn1_string_st ASN1_INTEGER;
}

namespace Fancy {

namespace SSL {

class FANCYSSL_API BigNum: public Fancy::SSL::Base {

public:
	typedef Fancy::ObjPtr<BigNum> ptr_t;

	static ptr_t newInstance(BIGNUM *_bn);
	static ptr_t newInstance(int i);
	static ptr_t newInstance(const Fancy::ArrayObject<uint8_t>::ptr_t &bin);
	static ptr_t newInstance(const Fancy::ArrayObject<int8_t>::ptr_t &bin);
	static ptr_t newInstance(const std::string &s);
	static ptr_t newHexInstance(const std::string &hex);

	BIGNUM *getBIGNUM() const;
	BIGNUM *dup() const;

	ptr_t add(int i) const;
	ptr_t sub(int i) const;

	ptr_t add(const ptr_t &a) const;
	ptr_t sub(const ptr_t &b) const;

	ptr_t mul(int i) const;
	ptr_t div(int i, int *rem) const;

	ptr_t mul(const ptr_t &a) const;
	ptr_t div(const ptr_t &a, ptr_t &rem) const;

	bool lessThan(const Fancy::Object &o) const;
	bool equals(const Fancy::Object &o) const;

	std::string toHexString() const;
	std::string toString() const;
	Fancy::ArrayObject<uint8_t>::ptr_t toBinary() const;

protected:
	BigNum(BIGNUM *_bn);
	virtual ~BigNum();
private:
	BIGNUM *bn;
};

}

}

#endif /* FANCYSSLBIGNUM_H_ */
