/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSSLCHANNELBIO_H_
#define FANCYSSLCHANNELBIO_H_

#include <fancysslbase.h>

// fwd decls from openssl.
extern "C" {
	struct bio_st;
	typedef struct bio_st BIO;
}

namespace Fancy {

class Channel;

namespace SSL {

/**
 * An adapter for Fancy::Channel to an OpenSSL BIO.
 */
class FANCYSSL_API ChannelBIO: public Fancy::SSL::Base {

	BIO *bio;

protected:
	ChannelBIO(BIO *_bio);
	virtual ~ChannelBIO();

public:

	typedef Fancy::ObjPtr<ChannelBIO> ptr_t;

	/**
	 * @return a BIO, which references the given channel. The caller is responsible
	 * for calling @c BIO_free() on the returned pointer, which will dereference
	 * the given channel.
	 */
	static BIO *newBIOInstance(const Fancy::ObjPtr<Fancy::Channel> &channel);

	/**
	 * Return a new @c ChannelBIO object, which manages a BIO referencing the
	 * given channel.
	 */
	static ptr_t newInstance(const Fancy::ObjPtr<Fancy::Channel> &channel);

	/**
	 * Return the BIO managed by this @c ChannelBIO instance. The caller
	 * must not call @c BIO_free on the returned pointer.
	 */
	BIO *getBIO() const;
};

}

}

#endif /* FANCYSSLCHANNELBIO_H_ */
