/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** Global logging stuff.
**
*****************************************************************************/

#ifndef FANCYSSL_LOG_H
#define FANCYSSL_LOG_H

#include <fancylog.h>
#include <fancysslbase.h>

// facility masks:
// facility "fancysslbase"
#define  FANCYSSL_LOG_BASE     (1<<0)
// facility "fancysslpkey"
#define  FANCYSSL_LOG_PKEY     (1<<1)
// facility "fancysslx509"
#define  FANCYSSL_LOG_X509     (1<<2)
// facility "fancysslpkcs7"
#define  FANCYSSL_LOG_PKCS7    (1<<3)
// facility "fancysslsock"
#define  FANCYSSL_LOG_SOCK     (1<<4)
// facility "fancyssldigest"
#define  FANCYSSL_LOG_DIGEST   (1<<5)
// facility "fancyssldes"
#define  FANCYSSL_LOG_DES      (1<<6)
// facility "fancysslrandom"
#define  FANCYSSL_LOG_RANDOM   (1<<7)
// facility "fancysslrandom"
#define  FANCYSSL_LOG_CIPHER   (1<<8)
// facility "fancysslpkcs12"
#define  FANCYSSL_LOG_PKCS12   (1<<9)
// facility "fancysslengine"
#define  FANCYSSL_LOG_ENGINE   (1<<10)
// facility "fancysslrsa"
#define  FANCYSSL_LOG_RSA      (1<<11)

namespace Fancy {
  namespace SSL {

    extern FANCYSSL_API fancylog_service *log_service;

    // parse for logging cmdline args and filter out recognized options.
    void FANCYSSL_API fancylog_parse_arg (int *argc, char **argv);

    inline void vfancylog (int facility,
                           int priority, const char *fmt, va_list ap)
      {
        va_list aq;
        va_copy(aq,ap);
        ::vfancylog(log_service,facility,priority,fmt,aq);
        va_end(aq);
      }

#ifdef __GNUG__
  inline void fancylog (int facility,
                        int priority, const char *fmt, ...)
    __attribute__((format(printf,3,4)));

#endif

    inline void fancylog (int facility,
                          int priority, const char *fmt, ...)
      {
        va_list ap;
        va_start(ap, fmt);
        vfancylog(facility,priority,fmt,ap);
        va_end(ap);
      }
  }
}

#endif // ! FANCYSSL_LOG_H
