/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/


#ifndef FANCYSSL_MGF_H
#define FANCYSSL_MGF_H

#include <fancysslbase.h>

namespace Fancy {
namespace SSL {

class Digest;

/**
 * A mask generation function for the OAEP algorithm as defined in
 * <a href="http://tools.ietf.org/html/rfc3447#appendix-B.2">Appendix B.2 of RFC 3447</a>.
 */
class FANCYSSL_API MGF : virtual public Fancy::Object
{

protected:
	MGF();
	virtual ~MGF();

public:

	typedef Fancy::ObjPtr<MGF> ptr_t;

	/**
	 * Return a new instance implementing
	 * the mask generation function no. 1 for the OAEP algorithm as defined in
	 * <a href="http://tools.ietf.org/html/rfc3447#appendix-B.2">Appendix B.2.1 of RFC 3447</a>.
	 */
	static ptr_t newMGF1(const Fancy::ObjPtr<Digest> &digest);

    /**
     * Generate <code>maskLen</code> mask bytes from a seed.
     * @param mgfSeed The seed value.
     * @param seedLen The number of seed bytes in <code>mgfSeed</code> to use.
     * @param ret The byte array where the result is being stored.
     * @param maskLen The number of octets to generate.
     */
    virtual void generateMask(const void *mgfSeed, size_t seedLen, void *ret, size_t maskLen) const = 0;

    /**
     * @return The message digest, which is used by this mask generator.
     */
    virtual Fancy::ObjPtr<Digest> getUnderlyingDigest() const = 0;
};
}
}

#endif
