/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2018 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
 **
 ** A class for handling PKCS12 archives
 **
 *****************************************************************************/

#ifndef FANCYSSL_PKCS12_H
#define FANCYSSL_PKCS12_H

#include <fancysslbase.h>

#include <string>


namespace Fancy {

//template<class T> class ArrayObject;
class List;

namespace SSL {

class PKey;
class X509Cert;


/**
 * A representation of a PKCS12 archive (or any structure with one PKey and one ore more X509Certs
 */
class FANCYSSL_API PKCS12Bag : public Base
{
public:

	typedef Fancy::ObjPtr<PKCS12Bag> ptr_t;

	/**
	 * Creates a new empty instance
	 */
	static ptr_t newInstance();

	/**
	 * Reads a PKCS12 file
	 * @param file   The path to the PKCS12 file.
	 * @param passwd The password used to decode the file or @c NULL, if the
	 *               the file is expected to be unencrypted.
	 * @return A new @c PKCS12 instance.
	 */
	static ptr_t readFile(
			const std::string & file,
			const std::string & passwd);

	/**
	 * Writes a PKCS12 file
	 * @param file   The path to the PKCS12 file.
	 * @param passwd The password used to decode the file or @c NULL, if the
	 *               the file is expected to be unencrypted.
	 * @return true if writing was successful
	 */
	bool writeFile(
			const std::string & filename,
			const std::string & password);


	Fancy::ObjPtr<PKey> getPKey();

	void setPKey(
			const Fancy::ObjPtr<PKey> & pKey);


	/*
	 * Returns main certificate
	 */
	Fancy::ObjPtr<Fancy::SSL::X509Cert> getCertificate();

	/*
	 * Sets main certificate
	 */
	void setCertificate(
			const Fancy::ObjPtr<Fancy::SSL::X509Cert> & certificate);


	/*
	 * Returns a list of X509Cert::ptr_t
	 */
	Fancy::ObjPtr<Fancy::List> getCAChain();

	/*
	 * Expects a list of X509Cert::ptr_t
	 */
	void setCAChain(
			const Fancy::ObjPtr<Fancy::List> & caChain);


private:
	PKCS12Bag();
	~PKCS12Bag();

	Fancy::ObjPtr<PKey> fancyPKey;
	Fancy::ObjPtr<X509Cert> fancyCertificate;
	Fancy::ObjPtr<Fancy::List> caChain;

};

}} // end namespaces

#endif // FANCYSSL_PKCS12_H
