/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/


#ifndef FANCYSSL_RANDOM_H
#define FANCYSSL_RANDOM_H

#include <fancysslbase.h>
#include <fancyarray.h>

namespace Fancy {
namespace SSL {

/**
 * Random number generator. It may be used as a singleton
 * throughout a single process.
 *
 * The actual implementation is actually statically calling
 * RAND_* of openssl, the object is actually present in order
 * to keep the openssl context open.
 */
class FANCYSSL_API Random : virtual public Fancy::Object
{

protected:

    Random ();
	virtual ~Random();

public:

	typedef Fancy::ObjPtr<Random> ptr_t;

	/**
	 * @return a new instancee for the given algorithm name.
	 */
	static ptr_t newInstance();

	/**
	 * Return whether the random number generator is fully seeded.
	 */
    virtual bool isSeeded() const = 0;

	/**
	 * Add @c len bytes with @c entropy of randomness.
	 * If the input is really unpredictable, you may choose
	 * entropy = len.
	 */
    virtual void addEntropy(const void *bytes, int len, int entropy) = 0;

    /**
	 * generate @c len random bytes.
	 */
	virtual void getBytes(void *buf, int len) = 0;

	/**
	 * Return random bytes as a fancy object array.
	 */
	virtual Fancy::ArrayObject<uint8_t>::ptr_t getBytes(int len);
};
}
}

#endif
