/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A class for handling OpenSSL X509 CRLs.
**
*****************************************************************************/

#ifndef FANCYSSL_X509CRL_H
#define FANCYSSL_X509CRL_H

#include <fancysslbase.h>

// fwd decls from openssl.
extern "C" {
  struct X509_crl_st;
  typedef struct X509_crl_st X509_CRL;
}

namespace Fancy {
  namespace SSL {

    /**
     * Manage an X509 certificate revocation list.
     */
    class FANCYSSL_API X509Crl : public Base
    {
    protected:
        X509Crl (X509_CRL *_x509_crl);
        ~X509Crl ();

    public:
    	typedef Fancy::ObjPtr<X509Crl> ptr_t;

        /**
          Aquires the given OpenSSL handle and automagically free the handle,
          if the object goes out of scope. The caller must not call \c X509_CRL_free
          on the supplied resource.
        */
    	static ptr_t newInstance(X509_CRL *x509_crl);

    	/**
    	 * Parse a PEM-encoded string.
    	 */
    	static ptr_t newInstance(const char *pem, int len=-1);

    	/**
    	 * Read a PEM-encoded file.
    	 */
    	static ptr_t readFile(const char *file);

    	/**
    	 * @return The OpenSSL handle managed by this X509Crl object.
    	 */
    	X509_CRL *getX509_CRL () const;

    	/**
    	 * @return A duplicate of the OpenSSL handle managed by this X509Crl object.
    	 *         The caller is responsible for calling @c X509_CRL_free on the
    	 *         returned pointer.
    	 */
    	X509_CRL *dup () const;

    	/**
    	 * @return a PEM-encoded version of this string.
    	 */
    	std::string getPEMString() const;

    private:
        X509Crl (const X509Crl &a);
        X509Crl &operator= (const X509Crl &a);

        X509_CRL *x509_crl;
    };
  }
}

#endif
