/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSSLX509EXT_H_
#define FANCYSSLX509EXT_H_

#include <fancysslbase.h>

extern "C" {
	struct X509_extension_st;
	typedef struct X509_extension_st X509_EXTENSION;
}

namespace Fancy {

namespace SSL {

class FANCYSSL_API X509Ext: public Fancy::SSL::Base {

public:
	typedef Fancy::ObjPtr<X509Ext> ptr_t;

	static ptr_t newInstance(X509_EXTENSION *_ext);

	/**
	 * Instantiate a new X509 extensin according to the object names and values in the
	 * <A HREF="http://www.openssl.org/docs/apps/x509v3_config.html">x509v3_config man page</A>.
	 *
	 * @param obj The name of the object, which identifies the extension.
	 * @param value The value of the extension.
	 * @return A new instance with the given object/value.
	 */
	static ptr_t newInstance(const std::string &obj, const std::string &value);

	X509_EXTENSION *getX509_EXTENSION() const;
	X509_EXTENSION *dup() const;

	std::string getObjID() const;
	std::string getObjShortName() const;
	std::string getObjLongName() const;
	std::string getValue() const;
	bool isCritical() const;

	void setCritical(bool c);

	std::string toString() const;

protected:
	X509Ext(X509_EXTENSION *_ext);
	virtual ~X509Ext();

private:
	X509_EXTENSION *ext;
};

}

}

#endif /* FANCYSSLX509EXT_H_ */
