/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A class for handling OpenSSL X509 requests (PKCS10).
**
*****************************************************************************/

#ifndef FANCYSSL_X509REQ_H
#define FANCYSSL_X509REQ_H

#include <fancysslbase.h>

// fwd decls from openssl.
extern "C" {
  struct X509_req_st;
  typedef struct X509_req_st X509_REQ;
}

namespace Fancy {
  namespace SSL {
    class X509Name;
    class PKey;

    class FANCYSSL_API X509Req : public Base
    {
    protected:
    	X509Req (X509_REQ *_x509_req);
    	~X509Req();

    public:
    	typedef Fancy::ObjPtr<X509Req> ptr_t;

    	static ptr_t newInstance(const char *pem, int len=-1);
    	static ptr_t newInstance(X509_REQ *x509_req);
    	static ptr_t readFile(const char *file);

    	static ptr_t generate(const Fancy::ObjPtr<X509Name> &dn, const Fancy::ObjPtr<PKey> &key);

    	X509_REQ *getX509_REQ () const;

    	std::string getPEMString() const;

    private:
        X509Req (const X509Req &a);
        X509Req &operator= (const X509Req &a);

        X509_REQ *x509_req;
    };
  }
}

#endif
